package com.zh.lingnuo.controller.company;


import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.fasterxml.jackson.annotation.JsonIgnore;
import com.zh.lingnuo.annotation.ExcelParam;
import com.zh.lingnuo.annotation.Token;

import com.zh.lingnuo.mapper.AnswererMapper;
import com.zh.lingnuo.model.Answerer;
import com.zh.lingnuo.model.Company;
import com.zh.lingnuo.model.CompanyQn;
import com.zh.lingnuo.model.CompanyQnAnswerer;
import com.zh.lingnuo.service.ICompanyQnAnswererService;
import com.zh.lingnuo.service.ICompanyQnService;
import com.zh.lingnuo.utils.PageUtil;
import com.zh.lingnuo.utils.result.Result;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.media.Schema;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.Data;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import org.springframework.web.multipart.MultipartFile;

import javax.validation.constraints.NotNull;

/**
 * <p>
 * 前端控制器
 * </p>
 *
 * @author zh
 * @since 2022-09-14
 */
@Tag(name = "答题人管理")
@RestController
@RequestMapping("/company_qn_answerer")
public class CompanyQnAnswererController {

    @Autowired
    private ICompanyQnService companyQnService;
    @Autowired
    private ICompanyQnAnswererService qnAnswererService;
    @Autowired
    private AnswererMapper answererMapper;

    @Data
    public static class paramsCompanyQnAnswerList {
        private int pageNum;
        private int pageSize;
        @Schema(title = "问卷id")
        @NotNull(message = "问卷id不能为空")
        private Integer qnId;
        @JsonIgnore(value = false)
        private Company userInfo;
    }

    @Operation(summary = "列表")
    @PostMapping("/list")
    @Token(requireUser = 1, level = {2})
    public Result list(@Validated @RequestBody paramsCompanyQnAnswerList params) {
        String companyErr = companyQnService.validCompany(params.getQnId(), params.getUserInfo().getId());
        if (null != companyErr) {
            return Result.err(companyErr);
        }
        return Result.success(answererMapper.answererPageList(PageUtil.SetPage(params.getPageNum(), params.getPageSize()), new QueryWrapper<Answerer>().eq("qn_id", params.getQnId()).orderByDesc("id")));
    }

    @Data
    public static class ExcelAnswerData {
        @ExcelParam(value = "姓名")
        private String name;
        @ExcelParam(value = "邮箱")
        private String mail;
        @ExcelParam(value = "联系方式")
        private String tel;
        @ExcelParam(value = "部门")
        private String department;
//        @ExcelParam(value = "工龄")
//        private Integer workAge;
    }

    @Operation(summary = "excel导入")
    @PostMapping("/addExcel")
    @Token(requireUser = 1, level = {2})
    public Result addExcel(@RequestParam("file") MultipartFile file, @RequestParam Integer id) {
//        String companyErr = manageQnService.validCompany(id,params.getUserInfo().getId());
//        if(null != companyErr){
//            return Result.err(companyErr);
//        }

        CompanyQn companyQn = companyQnService.getById(id);
        System.out.println(companyQn);
        System.out.println(companyQn == null);
        if (null == companyQn) {
            return Result.err("未知问卷");
        }
        return qnAnswererService.addExcel(file, id,companyQn.getCompanyId());
    }

    @Data
    public static class ParamQnAnswerAdd {
        @Schema(title = "问卷id")
        @NotNull(message = "问卷id不能为空")
        private Integer qnId;
        @Schema(title = "姓名")
        @NotNull(message = "姓名不能为空")
        private String name;
        @Schema(title = "邮箱")
        @NotNull(message = "邮箱不能为空")
        private String mail;
        @Schema(title = "联系方式")
        @NotNull(message = "联系方式不能为空")
        private String tel;
        @Schema(title = "部门")
        @NotNull(message = "部门不能为空")
        private String department;
        @Schema(title = "工龄")
        @NotNull(message = "工龄不能为空")
        private Integer workAge;
        @JsonIgnore(value = false)
        private Company userInfo;
    }

    @Operation(summary = "添加")
    @PostMapping("/add")
    @Token(requireUser = 1, level = {2})
    public Result add(@Validated @RequestBody ParamQnAnswerAdd params) {
        CompanyQn companyQn = companyQnService.getById(params.getQnId());
        if (null == companyQn) {
            return Result.err("未知问卷");
        }
        return qnAnswererService.add(params,companyQn.getCompanyId());
    }

    @Data
    public static class ParamCompanyQnAnswerUpd {
        @Schema(title = "答题人id")
        @NotNull(message = "答题人id不能为空")
        private Integer answererId;
        @Schema(title = "姓名")
        private String name;
        @Schema(title = "邮箱")
        private String mail;
        @Schema(title = "联系方式")
        private String tel;
        @Schema(title = "部门")
        private String department;
        @Schema(title = "工龄")
        private Integer workAge;
        @Schema(title = "密码")
        private String password;
        @JsonIgnore(value = false)
        private Company userInfo;
    }

    @Operation(summary = "修改")
    @PostMapping("/upd")
    @Token(requireUser = 1, level = {2})
    public Result upd(@Validated @RequestBody ParamCompanyQnAnswerUpd params) {
        return qnAnswererService.upd(params);
    }

    @Data
    public static class ParamCompanyQnAnswerDel {
        @Schema(title = "答题人id")
        @NotNull(message = "答题人id不能为空")
        private Integer answererId;
        @Schema(title = "问卷id")
        @NotNull(message = "问卷id不能为空")
        private Integer qnId;
        @JsonIgnore(value = false)
        private Company userInfo;
    }

    @Operation(summary = "删除")
    @PostMapping("/del")
    @Token(requireUser = 1, level = {2})
    public Result del(@Validated @RequestBody ParamCompanyQnAnswerDel params) {
        if (!qnAnswererService.remove(new QueryWrapper<CompanyQnAnswerer>().eq("qn_id",params.getQnId()).eq("answerer_id",params.getAnswererId()))) {
            return Result.err("删除无效");
        }
        return Result.success("");
    }

}
