package com.zh.lingnuo.controller.company;


import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.fasterxml.jackson.annotation.JsonIgnore;
import com.zh.lingnuo.annotation.Token;
import com.zh.lingnuo.model.Company;
import com.zh.lingnuo.model.CompanyQn;
import com.zh.lingnuo.service.ICompanyQnService;
import com.zh.lingnuo.service.ICompanyQnTitleService;
import com.zh.lingnuo.utils.result.Result;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.media.Schema;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.Data;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;

import org.springframework.web.bind.annotation.RestController;

import javax.validation.constraints.Max;
import javax.validation.constraints.Min;
import javax.validation.constraints.NotEmpty;
import javax.validation.constraints.NotNull;
import java.time.LocalDateTime;
import java.util.List;

/**
 * <p>
 * 前端控制器
 * </p>
 *
 * @author zh
 * @since 2022-09-19
 */
@Tag(name = "问卷题目管理")
@RestController
@RequestMapping("/company_qn_title")
public class CompanyQnTitleController {

    @Autowired
    private ICompanyQnService companyQnService;
    @Autowired
    private ICompanyQnTitleService companyQnTitleService;

    @Data
    public static class paramsQnTitleList {
//        private int pageNum;
//        private int pageSize;
        @Schema(title = "问卷id")
        @NotNull(message = "问卷id不能为空")
        private Integer qnId;
        @JsonIgnore(value = false)
        private Company userInfo;
    }

    @Data
    public static class ResultQnTitleList {
        private Integer id;
        private Integer qnId;
        private Integer titleId;
        private String title;
        private Integer kind;
        private String kindName;
        private Integer groupId;
        private String groupName;
        private JSONObject options;
        private LocalDateTime updatedAt;
        private LocalDateTime createdAt;
    }

    @Operation(summary = "列表")
    @PostMapping("/list")
    @Token(requireUser = 1, level = {2})
    public Result<List<ResultQnTitleList>> list(@Validated @RequestBody paramsQnTitleList params) {
        String companyErr = companyQnService.validCompany(params.getQnId(),params.getUserInfo().getId());
       if(null != companyErr){
           return Result.err(companyErr);
       }
        return companyQnTitleService.qnTitleList(params.getQnId());
    }

    @Data
    public static class paramsQnTitleGroupAdd {
        @Schema(title = "问卷id")
        @NotNull(message = "问卷id不能为空")
        private Integer qnId;
        @NotNull(message = "题目列表不能为空")
        private paramsQnTitleAdd [] titleGroup;
        @JsonIgnore(value = false)
        private Company userInfo;
    }
    @Data
    public static class paramsQnTitleAdd{
        @Schema(title = "标题")
        @NotNull(message = "标题不能为空")
        @NotEmpty(message = "标题不能为空")
        private String title;
        @Schema(title = "题型")
        @NotNull(message = "题型不能为空")
        @Min(value = 1, message = "类型范围为1 2 3")
        @Max(value = 3, message = "类型范围为1 2 3")
        private Integer kind;
        @Schema(title = "分组")
        @NotNull(message = "分组不能为空")
        @Min(value = 1, message = "分组:1=个人 2=调研")
        @Max(value = 2, message = "分组:1=个人 2=调研")
        private Integer groupId;
        @Schema(title = "选项")
        private String[] options;
    }

//    @Operation(summary = "新增")
//    @PostMapping("/add")
//    @Token(requireUser = 1, level = {2})
//    public Result<CompanyQn> add(@Validated @RequestBody paramsQnTitleGroupAdd params) {
//        String companyErr = companyQnService.validCompany(params.getQnId(),params.getUserInfo().getId());
//        if(null != companyErr){
//            return Result.err(companyErr);
//        }
//        return companyQnTitleService.qnTitleAdd(params);
//    }

    @Data
    public static class paramsQnTitleAddModel {
        @Schema(title = "问卷id")
        @NotNull(message = "问卷id不能为空")
        private Integer qnId;
        @Schema(title = "题型库id")
        private Integer[] titleIds;
        @JsonIgnore(value = false)
        private Company userInfo;
    }

    @Operation(summary = "添入题库题目")
    @PostMapping("/addModel")
    @Token(requireUser = 1, level = {2})
    public Result<CompanyQn> addModel(@Validated @RequestBody paramsQnTitleAddModel params) {
        String companyErr = companyQnService.validCompany(params.getQnId(),params.getUserInfo().getId());
        if(null != companyErr){
            return Result.err(companyErr);
        }
        return companyQnTitleService.qnTitleAddModel(params);
    }

    @Data
    public static class paramsQnTitleUpd {
        @Schema(title = "id")
        @NotNull(message = "id不能为空")
        private Integer id;
        @Schema(title = "标题")
        private String title;
        @Schema(title = "选项")
        private String[] options;
        @JsonIgnore(value = false)
        private Company userInfo;
    }

//    @Operation(summary = "修改")
//    @PostMapping("/upd")
//    @Token(requireUser = 1, level = {2})
//    public Result<CompanyQn> upd(@Validated @RequestBody paramsQnTitleUpd params) {
//        return companyQnTitleService.qnTitleUpd(params);
//    }

    @Data
    public static class paramsQnTitleDel {
        @Schema(title = "id")
        @NotNull(message = "id不能为空")
        private Integer id;
        @JsonIgnore(value = false)
        private Company userInfo;
    }

    @Operation(summary = "删除")
    @PostMapping("/del")
    @Token(requireUser = 1, level = {2})
    public Result<CompanyQn> del(@Validated @RequestBody paramsQnTitleDel params) {
        return companyQnTitleService.qnTitleDel(params);
    }

}
