package com.zh.lingnuo.controller.manage;


import com.baomidou.mybatisplus.core.metadata.IPage;
import com.fasterxml.jackson.annotation.JsonFormat;
import com.zh.lingnuo.annotation.Token;
import com.zh.lingnuo.model.CompanyQn;
import com.zh.lingnuo.service.ICompanyService;
import com.zh.lingnuo.utils.result.Result;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.media.Schema;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.Data;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.validation.constraints.Max;
import javax.validation.constraints.Min;
import javax.validation.constraints.NotEmpty;
import javax.validation.constraints.NotNull;
import java.time.LocalDateTime;
import java.util.List;

/**
 * <p>
 * 前端控制器
 * </p>
 *
 * @author zh
 * @since 2022-09-13
 */
@Tag(name = "企业管理")
@RestController
@RequestMapping("/company")
public class CompanyController {

    @Autowired
    private ICompanyService companyService;

    @Data
    public static class paramsCompanyList {
        private int pageNum;
        private int pageSize;
        @Schema(title = "企业名称")
        private String name;
        @Schema(title = "更新时间起")
        @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
        private LocalDateTime updateTimeStart;
        @Schema(title = "更新时间止")
        @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
        private LocalDateTime updateTimeEnd;
        @Schema(title = "创建时间起")
        @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
        private LocalDateTime createTimeStart;
        @Schema(title = "创建时间止")
        @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
        private LocalDateTime createTimeEnd;
    }

    @Data
    public static class resultCompany {
        private Integer id;
        @Schema(title = "名称")
        private String name;
        @Schema(title = "邮箱")
        private String mail;
        @Schema(title = "联系方式")
        private String tel;
        @Schema(title = "状态")
        private Integer status;
        @Schema(title = "状态名称")
        private String statusName;
        @Schema(title = "创建时间")
        private LocalDateTime createdAt;
        @Schema(title = "更新时间")
        private LocalDateTime updatedAt;
        @Schema(title = "问卷列表")
        private List<CompanyQn> qnList;
    }

    @Operation(summary = "列表")
    @PostMapping("/list")
    @Token(requireUser=1,level = {1})
    public Result<IPage<resultCompany>> list(@Validated @RequestBody paramsCompanyList params) {
        return companyService.CompanyList(params);
    }

    @Data
    public static class paramsCompanyAdd {
        @Schema(title = "企业名称")
        @NotNull(message = "企业名称不能为空")
        @NotEmpty(message = "企业名称不能为空")
        private String name;
        @Schema(title = "邮箱")
        @NotNull(message = "邮箱不能为空")
        @NotEmpty(message = "企业名称不能为空")
        private String mail;
        @Schema(title = "联系方式")
        @NotNull(message = "联系方式不能为空")
        @NotEmpty(message = "企业名称不能为空")
        private String tel;
        @Schema(title = "密码")
        @NotNull(message = "密码不能为空")
        @NotEmpty(message = "企业名称不能为空")
        private String password;
    }

    @Operation(summary = "添加")
    @PostMapping("/add")
    @Token(requireUser=1,level = {1})
    public Result add(@Validated @RequestBody paramsCompanyAdd params) {
        return companyService.CompanyAdd(params);
    }

    @Data
    public static class paramsCompanyInfo {
        @Schema(title = "企业id")
        @NotNull(message = "企业id不能为空")
        private Integer id;
    }

    @Operation(summary = "详情")
    @PostMapping("/info")
    @Token(requireUser=1,level = {1})
    public Result<resultCompany> info(@Validated @RequestBody paramsCompanyInfo params) {
        return companyService.CompanyInfo(params);
    }

    @Data
    public static class paramsCompanyAble {
        @Schema(title = "企业id")
        @NotNull(message = "企业id不能为空")
        private Integer id;
        @Schema(title = "操作",description = "1启用;0停用")
        @NotNull(message = "操作不能为空")
        @Min(value = 0, message = "类型范围为0 1")
        @Max(value = 1, message = "类型范围为0 1")
        private Integer status;
    }

    @Operation(summary = "启用停用")
    @PostMapping("/able")
    @Token(requireUser=1,level = {1})
    public Result able(@Validated @RequestBody paramsCompanyAble params) {
        return companyService.CompanyAble(params);
    }

}
