package com.zh.lingnuo.controller.manage;


import com.zh.lingnuo.annotation.Token;
import com.zh.lingnuo.service.IUserService;
import com.zh.lingnuo.utils.result.Result;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.media.Schema;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.Data;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.constraints.Max;
import javax.validation.constraints.Min;
import javax.validation.constraints.NotNull;

/**
 * <p>
 *  前端控制器
 * </p>
 *
 * @author zh
 * @since 2022-09-08
 */
@Tag(name = "用户")
@RestController
@RequestMapping("/user")
public class UserController {

    @Autowired
    private IUserService userService;

    @Data
    public static class ParamsAddUser{
        @Schema(title = "名称")
        @NotNull(message = "名称不能为空")
        private String name;
        @Schema(title = "联系方式")
        @NotNull(message = "联系方式不能为空")
        private String phone;
        @Schema(title = "邮箱")
        @NotNull(message = "邮箱不能为空")
        private String mail;
        @Schema(title = "类型")
        @NotNull(message = "类型不能为空")
        @Min(value = 1, message = "类型范围为1 2 3")
        @Max(value = 3, message = "类型范围为1 2 3")
        private int level;
        @Schema(title = "密码")
        @NotNull(message = "密码不能为空")
        private String password;
    }

    @Operation(summary = "添加用户")
    @PostMapping("/add")
    @Token(requireUser = 1, level = {1})
    public Result add(@Validated @RequestBody ParamsAddUser params){
        return   userService.addUser(params);
    }
}
