package com.zh.lingnuo.serviceImpl;

import com.alibaba.fastjson.JSONObject;
import com.aliyun.dysmsapi20170525.Client;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.zh.lingnuo.controller.company.CompanyQnController;
import com.zh.lingnuo.controller.company.CompanyQnController.ResultCompanyQnAnswer;
import com.zh.lingnuo.controller.company.CompanyQnController.ResultCompanyQn;
import com.zh.lingnuo.controller.company.CompanyQnController.CountResult;
import com.zh.lingnuo.controller.manage.ManageCompanyQnController;
import com.zh.lingnuo.mapper.AnswererMapper;
import com.zh.lingnuo.mapper.CompanyQnAnswererMapper;
import com.zh.lingnuo.model.Answerer;
import com.zh.lingnuo.model.Company;
import com.zh.lingnuo.model.CompanyQn;
import com.zh.lingnuo.mapper.CompanyQnMapper;
import com.zh.lingnuo.model.CompanyQnAnswerer;
import com.zh.lingnuo.service.ICompanyQnAnswererService;
import com.zh.lingnuo.service.ICompanyQnService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.zh.lingnuo.utils.Common;
import com.zh.lingnuo.utils.FuncRes;
import com.zh.lingnuo.utils.PageUtil;
import com.zh.lingnuo.utils.SmsService;
import com.zh.lingnuo.utils.result.Result;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author zh
 * @since 2022-09-13
 */
@Service
public class CompanyQnServiceImpl extends ServiceImpl<CompanyQnMapper, CompanyQn> implements ICompanyQnService {

    @Value("${web.webUrl}")
    private String webUrl;

    @Value("${aliyun.sms.signName}")
    private String signName;

    @Value("${aliyun.sms.templateCode}")
    private String templateCode;

    @Autowired
    private ICompanyQnAnswererService companyQnAnswererService;
    @Autowired
    MailServiceImpl mailServiceImpl;
    @Autowired
    private SmsService smsService;
    @Autowired
    private CompanyQnMapper companyQnMapper;
    @Autowired
    private AnswererMapper answererMapper;
    @Autowired
    private CompanyQnAnswererMapper companyQnAnswererMapper;

    // 管理端问卷列表
    public Result<IPage<CompanyQn>> manageQnList(ManageCompanyQnController.paramsManageCompanyQnList params) {
        QueryWrapper<CompanyQn> qw = new QueryWrapper<>();
        qw.like(StringUtils.isNotBlank(params.getTitle()), "t1.title", params.getTitle())
            .like(StringUtils.isNotBlank(params.getCompanyName()), "t2.name", params.getCompanyName())
            .eq(null != params.getStatus(), "t1.status", params.getStatus())
            .ge(null != params.getCreateTimeStart(), "t1.created_at", params.getCreateTimeStart())
            .le(null != params.getCreateTimeEnd(), "t1.created_at", params.getCreateTimeEnd())
            .ge(null != params.getUpdateTimeStart(), "t1.updated_at", params.getUpdateTimeStart())
            .le(null != params.getUpdateTimeEnd(), "t1.updated_at", params.getUpdateTimeEnd());
        IPage page = companyQnMapper.companyQnList(PageUtil.SetPage(params.getPageNum(), params.getPageSize()), qw);
        return Result.success(page);
    }

    // 列表
    public Result<IPage<CompanyQn>> qnList(CompanyQnController.paramsQnList params) {
        QueryWrapper<CompanyQn> qw = new QueryWrapper<>();
        qw.like(StringUtils.isNotBlank(params.getTitle()), "t1.title", params.getTitle())
            .eq("t1.company_id", params.getUserInfo().getId())
            .eq(null != params.getStatus(), "t1.status", params.getStatus())
            .ge(null != params.getCreateTimeStart(), "t1.created_at", params.getCreateTimeStart())
            .le(null != params.getCreateTimeEnd(), "t1.created_at", params.getCreateTimeEnd())
            .ge(null != params.getUpdateTimeStart(), "t1.updated_at", params.getUpdateTimeStart())
            .le(null != params.getUpdateTimeEnd(), "t1.updated_at", params.getUpdateTimeEnd());
        IPage page = companyQnMapper.companyQnList(PageUtil.SetPage(params.getPageNum(), params.getPageSize()), qw);
        return Result.success(page);
    }

    // 新增
    public Result qnAdd(CompanyQnController.paramsQnAdd params) {
        CompanyQn companyQn = new CompanyQn()
            .setCompanyId(params.getUserInfo().getId())
            .setTitle(params.getTitle())
            .setSubtitle(params.getSubtitle())
            .setIsSecret(params.getIsSecret());
        if (!super.save(companyQn)) {
            return Result.err("保存失败");
        }
        return Result.success(companyQn);
    }

    // 修改基础信息
    public Result qnUpd(CompanyQnController.paramsQnUpd params) {
        if (!super.update(new CompanyQn(), new UpdateWrapper<CompanyQn>()
            .eq("id", params.getQnId())
            .set(StringUtils.isNotBlank(params.getTitle()), "title", params.getTitle())
            .set(StringUtils.isNotBlank(params.getSubtitle()), "subtitle", params.getSubtitle()))) {
            return Result.err("无效操作");
        }
        return Result.success("");
    }

    // 发布
    public Result release(CompanyQn qn) {
        int qnId = qn.getId();
        List<Answerer> answerers = answererMapper.answererList(qnId);
        if (answerers.size() == 0) {
            return Result.err("答题人为空");
        }
        super.update(new UpdateWrapper<CompanyQn>().eq("id", qnId).eq("status", CompanyQn.CompanyQnStatusNotRelease).set("status", CompanyQn.CompanyQnStatusRelease));
        return Result.success("发布成功");
    }

    // 全量发送邮件
    public Result groupSendMail(int qnId,Company company) {
        CompanyQn qn = super.getById(qnId);
        String companyName = company.getName();
        String isSecret = CompanyQn.getIsSecretDesc(qn.getIsSecret());
        List<Answerer> answerers = answererMapper.answererList(qn.getId());
        List<String> illegalMailList = new ArrayList();
        List<String> illegalTelList = new ArrayList();
        System.out.println("发送短信邮件开始");
        for (Answerer answerer : answerers) {
            String mail = answerer.getMail();
            String usrName = answerer.getMail();
            String password = answerer.getTel();
            if (null != mail) {
//               String content = "请您用提供的账号密码登录该平台，并回答相应问题。\n"
//                   +"问卷地址："+webUrl+"\n"
//                   +"账号："+usrName+"\n"
//                   +"密码："+password+"\n"
//                   +"注意：本次调研采用【"+isSecret+"】形式。在最后的群体报告中，您的个人信息均会隐去。您的个人报告只有您自己能看到。";
                String content = "欢迎您参加" + companyName + "发起的组织网络问卷调查。\n"
                    + "问卷填写地址：" + webUrl + "\n"
                    + "账号：" + usrName + "\n"
                    + "密码" + password + "\n";
                if (!mailServiceImpl.sendMail(mail, signName, content)) {
                    illegalMailList.add(answerer.getName());
                }
            }
            String tel = answerer.getTel();
            Client client = null;
            try {
                client = smsService.createClient();
            } catch (Exception e) {
                System.out.println("注册smsClient错误");
            }
            if (null != client) {
                if (Common.telRegex(tel)) {
                    JSONObject obj = new JSONObject();
                    obj.put("company", companyName);
                    obj.put("usrname", usrName);
                    obj.put("password", password);
                    FuncRes funcRes = smsService.smsRequest(client, signName, templateCode, tel, obj.toString());
//                    System.out.println(obj);
//                    System.out.println(funcRes.getMsg());
                    if (!funcRes.isSuccess()) {
                        illegalTelList.add(answerer.getName());
                    }
                }
            }
        }
        String msg = "发布成功";
        if (illegalMailList.size() > 0) {
            msg = msg + "," + illegalMailList.toString() + "邮箱错误";
        }
        if (illegalTelList.size() > 0) {
            msg = msg + "," + illegalTelList.toString() + "手机号错误";
        }
        return Result.success("", msg + "!");
    }


    // 停止问卷
    public Result end(int qnId) {
        if (!super.update(new UpdateWrapper<CompanyQn>().eq("id", qnId).eq("status", CompanyQn.CompanyQnStatusRelease).set("status", CompanyQn.CompanyQnStatusEnd))) {
            return Result.err("操作错误");
        }
        return Result.success("");
    }

    // 问卷结果
    public Result result(int qnId) {
        List<CompanyQnAnswererMapper.MapperResultQnAnswererAll> list = companyQnAnswererMapper.qnAnswererList(new QueryWrapper<CompanyQnAnswerer>()
            .eq("qn_id", qnId).orderByDesc("status"));
        ResultCompanyQn res = new ResultCompanyQn();

        List<ResultCompanyQnAnswer> resList = new ArrayList<>();
        for (CompanyQnAnswererMapper.MapperResultQnAnswererAll i : list) {
            ResultCompanyQnAnswer r = new ResultCompanyQnAnswer();
            BeanUtils.copyProperties(i, r);
            r.setId(i.getAnswererId());
            r.setStatusName(CompanyQnAnswerer.getStatusName(i.getStatus()));
            resList.add(r);
        }
        int completedNum = 0;
        int progressingNum = 0;
        int waitingNum = 0;
        for (CompanyQnAnswererMapper.MapperResultQnAnswererAll answerer : list) {
            switch (answerer.getStatus()) {
                case 0:
                    waitingNum += 1;
                    break;
                case 1:
                    progressingNum += 1;
                    break;
                case 2:
                    completedNum += 1;
                    break;
                default:
                    break;
            }
        }
        res.setCount(new CountResult()
            .setCompletedNum(completedNum)
            .setProgressingNum(progressingNum)
            .setWaitingNum(waitingNum));
        res.setList(resList);
        return Result.success(res);
    }

    // 校验问卷所属企业
    public String validCompany(int qnId, int companyId) {
        CompanyQn companyQn = super.getById(qnId);
        if (null == companyQn) {
            return "未知问卷";
        }
        if (companyQn.getCompanyId() != companyId) {
            return "错误问卷";
        }
        return null;
    }
}
