package com.zh.lingnuo.serviceImpl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.zh.lingnuo.controller.UserTokenController;
import com.zh.lingnuo.model.*;
import com.zh.lingnuo.mapper.UserTokenMapper;
import com.zh.lingnuo.service.*;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.zh.lingnuo.utils.Common;
import com.zh.lingnuo.utils.result.Result;
import lombok.Data;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author zh
 * @since 2022-09-08
 */
@Service
public class UserTokenServiceImpl extends ServiceImpl<UserTokenMapper, UserToken> implements IUserTokenService {

    @Autowired
    private ICompanyService companyService;
    @Autowired
    private IAnswererService answererService;
    @Autowired
    private IUserService userService;


    @Data
    private class UserInfo{
        private Integer id;
        private String name;
        private String salt;
        private String password;
    }

    // 登录
    public Result<UserTokenController.ResultsLoginUser> login(UserTokenController.ParamsLoginUser params) {
        UserInfo userInfo = new UserInfo();
        switch (params.getKind()){
            case 1:
                User user = userService.getOne(new QueryWrapper<User>().eq("name", params.getName()));
                if(null == user){
                    return Result.err("账号错误");
                }
                BeanUtils.copyProperties(user,userInfo);
                break;
            case 2:
                Company company = companyService.getOne(new QueryWrapper<Company>()
                    .eq("name", params.getName()).or().eq("mail",params.getName()));
                if(null == company){
                    return Result.err("账号错误");
                }
                BeanUtils.copyProperties(company,userInfo);
                break;
            case 3:
                Answerer answerer = answererService.getOne(new QueryWrapper<Answerer>()
                    .eq("mail", params.getName()).last("limit 1"));
                if(null == answerer){
                    return Result.err("账号错误");
                }
                BeanUtils.copyProperties(answerer,userInfo);
                break;
            default:
                return Result.err("账号类型错误");
        }

        if (userInfo == null) {
            return Result.err("账号错误");
        }
        if (!Common.md5(userInfo.getSalt() + Common.md5(params.getPassword() + userInfo.getSalt())).equals(userInfo.getPassword())) {
            return Result.err("密码错误");
        }
        String token = Common.getRandomString(64, "");
        UserToken userToken = super.getOne(new QueryWrapper<UserToken>().eq("kind",params.getKind()).eq("user_id", userInfo.getId()).last("limit 1"));
        if (userToken != null) {
            super.update(new UserToken(),new UpdateWrapper<UserToken>().set("token", token).set("kind",params.getKind()).eq("user_id", userInfo.getId()));
        } else {
            super.save(new UserToken().setToken(token).setKind(params.getKind()).setUserId(userInfo.getId()));
        }
        UserTokenController.ResultsLoginUser result = new UserTokenController.ResultsLoginUser()
            .setName(userInfo.getName())
            .setKind(params.getKind())
            .setKindName(UserToken.getKindName(params.getKind()))
            .setToken(token);
        return Result.success(result);
    }

    // 退出登录
    public Result logout(String token){
        super.update(new UserToken(),new UpdateWrapper<UserToken>().set("token","").eq("token",token));
        return Result.success("");
    }

}
